@@ -1,12 +1,15 @@
 from nose import SkipTest, with_setup
 from nose.plugins.attrib import attr
-from numpy.testing import assert_equal, assert_raises, assert_allclose
+from numpy.testing import assert_equal, assert_raises
 
 from brian2 import *
 from brian2.core.functions import timestep
 from brian2.parsing.sympytools import str_to_sympy, sympy_to_str
 from brian2.utils.logger import catch_logs
 from brian2.devices.device import reinit_devices
+from brian2.tests.utils import assert_allclose
+from brian2.codegen.generators import CodeGenerator
+from brian2.codegen.codeobject import CodeObject
 
 @attr('codegen-independent')
 def test_constants_sympy():
@@ -114,8 +117,8 @@ def test_clip():
     run(defaultclock.dt)
     assert_equal(s_mon.clipexpr1.flatten(), [0, 1, 0, 1])
     assert_equal(s_mon.clipexpr2.flatten(), [0, 1, 0, 1])
-    assert_equal(s_mon.clipexpr3.flatten(), [0, 1, 0, 1])
-    assert_equal(s_mon.clipexpr4.flatten(), [0, 1, -0.5, 1.5])
+    assert_allclose(s_mon.clipexpr3.flatten(), [0, 1, 0, 1])
+    assert_allclose(s_mon.clipexpr4.flatten(), [0, 1, -0.5, 1.5])
 
 
 @attr('standalone-compatible')
@@ -142,16 +145,27 @@ def test_timestep_function():
     # Check that multiples of dt end up in the correct time step
     t = np.arange(100000)*dt
     assert_equal(timestep(t, dt), np.arange(100000))
-    # Check that inf is handled correctly
-    t = np.array([-np.inf, np.inf])
-    ts = timestep(t, dt)
-    assert ts[0] < -1e9
-    assert ts[1] > 1e9
 
     # Scalar values should stay scalar
     ts = timestep(0.0005, 0.0001)
     assert np.isscalar(ts) and ts == 5
 
+    # Length-1 arrays should stay arrays
+    ts = timestep(np.array([0.0005]), 0.0001)
+    assert ts.shape == (1,) and ts == 5
+
+
+@attr('standalone-compatible')
+@with_setup(teardown=reinit_devices)
+def test_timestep_function_during_run():
+    group = NeuronGroup(2, '''ref_t : second
+                              ts = timestep(ref_t, dt) + timestep(t, dt) : integer''')
+    group.ref_t = [-1e4*second, 5*defaultclock.dt]
+    mon = StateMonitor(group, 'ts', record=True)
+    run(5*defaultclock.dt)
+    assert all(mon.ts[0] <= -1e4)
+    assert_equal(mon.ts[1], [5, 6, 7, 8, 9])
+
 
 @attr('standalone-compatible')
 @with_setup(teardown=reinit_devices)
@@ -178,7 +192,7 @@ def test_user_defined_function():
     G.variable = test_array
     mon = StateMonitor(G, 'func', record=True)
     run(default_dt)
-    assert_equal(np.sin(test_array), mon.func_.flatten())
+    assert_allclose(np.sin(test_array), mon.func_.flatten())
 
 
 @with_setup(teardown=reinit_devices)
@@ -238,7 +252,7 @@ def test_simple_user_defined_function():
     net = Network(G, mon)
     net.run(default_dt)
 
-    assert_equal(np.sin(test_array), mon.func_.flatten())
+    assert_allclose(np.sin(test_array), mon.func_.flatten())
 
     # Check that it raises an error for C++
     try:
@@ -659,6 +673,7 @@ if __name__ == '__main__':
             test_clip,
             test_bool_to_int,
             test_timestep_function,
+            test_timestep_function_during_run,
             test_user_defined_function,
             test_user_defined_function_units,
             test_simple_user_defined_function,
